Imports System.ComponentModel
Imports System.xml

''' <summary>
''' Validate that at least one out of a list of selected controls has a value (any value).
''' </summary>
''' <remarks>Can accept checkbox, checkboxlist, radioboxlist, and calendar in addition to the
''' other normal validation controls.  Except for the calendar control, client-side validation
''' is supported.  Checkbox and radio buttons are said to have a value if they are checked.  Other
''' controls are said to have a value if their length is > 0 after trimming leading and trailing
''' white space.</remarks>
Partial Public Class AtLeastOneOfValidator
    Inherits BaseValidator

    Protected Overrides Function EvaluateIsValid() As Boolean
        Return Not IterateControlsToValidate(AddressOf ControlIsBlank) 'if any control is not blank, we are valid
    End Function

    Protected Overrides Function ControlPropertiesValid() As Boolean
        'make sure we have controls
        If (ControlsToValidate.Length = 0) Then
            Throw New HttpException(String.Format("{0} has no controls set for validation.", Me.ID))
        End If

        'make sure all the entries are valid control ids
        If Not IterateControlsToValidate(AddressOf ControlExists) Then
            Throw New HttpException(String.Format("Item '{0}' entry in the {1}.ControlsToValidate property is not a valid control.", ID, Me.ID))
        End If

        'every thing is okay
        Return True
    End Function

    Private Function ControlIsBlank(ByVal ControlID As String) As Boolean
        Dim ctrl As WebControl = Parent.FindControl(ControlID)
        If TypeOf ctrl Is CheckBox Then
            If DirectCast(ctrl, CheckBox).Checked Then Return False
        ElseIf TypeOf ctrl Is RadioButton Then
            If DirectCast(ctrl, RadioButton).Checked Then Return False
        ElseIf TypeOf ctrl Is CheckBoxList Then
            If DirectCast(ctrl, CheckBoxList).SelectedItem IsNot Nothing Then Return False
        ElseIf TypeOf ctrl Is RadioButtonList Then
            If DirectCast(ctrl, RadioButtonList).SelectedItem IsNot Nothing Then Return False
        ElseIf TypeOf ctrl Is Calendar Then
            If DirectCast(ctrl, Calendar).SelectedDates.Count > 0 Then Return False
        Else 'listbox, dropdownlist, and textbox
            If GetControlValidationValue(ControlID).Trim.Length > 0 Then Return False
        End If
        Return True
    End Function

    Private Function ControlExists(ByVal ControlID As String) As Boolean
        If Parent.FindControl(ControlID) IsNot Nothing Then Return True Else Return False
    End Function

    Private Delegate Function CheckControlById(ByVal ControlID As String) As Boolean
    Private Function IterateControlsToValidate(ByVal CheckControl As CheckControlById) As Boolean
        Dim delim() As Char = {","c}
        Dim ids() As String = ControlsToValidate.Split(delim, StringSplitOptions.RemoveEmptyEntries)
        For Each id As String In ids
            If Not CheckControl(id.Trim) Then Return False
        Next id
        Return True
    End Function

#Region "Controls To Validate"
    ''' <summary>
    ''' ControlToValidate is inherited from BaseValidator.  In this implementation it is managed
    ''' by ControlsToValidate and ControlToValidate(n) properties, and should NOT be edited directly.
    ''' </summary>
    ''' <remarks>Do not change this property directly.</remarks>
    <Browsable(False)> _
    <EditorBrowsable(EditorBrowsableState.Never)> _
    Public Shadows ReadOnly Property ControlToValidate() As String
        Get
            Return ""
        End Get
    End Property

    Private _ControlToValidate(9) As String

    ''' <summary>
    ''' Comma-delimited list of controls to validate.
    ''' </summary>
    ''' <remarks>It is okay to set this directly, but if you do and then set any of the 
    ''' ControlToValidate(X) properties it will be overwritten.  Likewise, setting this
    ''' property directly will remove anything set in the other properties.</remarks>
    <Browsable(True)> _
    Public Property ControlsToValidate() As String
        Get
            Return _ControlsToValidate
        End Get
        Set(ByVal value As String)
            'set directly in the designer. Clear any of the other control properties
            For i As Integer = 0 To _ControlToValidate.Length - 1
                _ControlToValidate(i) = ""
            Next i

            _ControlsToValidate = value
            SetBaseControlToValidate()
        End Set
    End Property
    Dim _ControlsToValidate As String

    ''' <summary>
    ''' First control to validate.
    ''' </summary>
    <Browsable(True)> _
    <TypeConverterAttribute(GetType(ValidatedControlConverterPlus))> _
    Public Property ControlToValidate0() As String
        Get
            Return _ControlToValidate(0)
        End Get
        Set(ByVal value As String)
            _ControlToValidate(0) = value
            BuildControlsToValidateString()
        End Set
    End Property

    ''' <summary>
    ''' Second control to validate.
    ''' </summary>
    <Browsable(True)> _
    <TypeConverterAttribute(GetType(ValidatedControlConverterPlus))> _
    Public Property ControlToValidate1() As String
        Get
            Return _ControlToValidate(1)
        End Get
        Set(ByVal value As String)
            _ControlToValidate(1) = value
            BuildControlsToValidateString()
        End Set
    End Property

    ''' <summary>
    ''' Third control to validate.
    ''' </summary>
    <Browsable(True)> _
    <TypeConverterAttribute(GetType(ValidatedControlConverterPlus))> _
    Public Property ControlToValidate2() As String
        Get
            Return _ControlToValidate(2)
        End Get
        Set(ByVal value As String)
            _ControlToValidate(2) = value
            BuildControlsToValidateString()
        End Set
    End Property

    ''' <summary>
    ''' Fourth control to validate.
    ''' </summary>
    <Browsable(True)> _
    <TypeConverterAttribute(GetType(ValidatedControlConverterPlus))> _
    Public Property ControlToValidate3() As String
        Get
            Return _ControlToValidate(3)
        End Get
        Set(ByVal value As String)
            _ControlToValidate(3) = value
            BuildControlsToValidateString()
        End Set
    End Property

    ''' <summary>
    ''' Fifth control to validate.
    ''' </summary>
    <Browsable(True)> _
    <TypeConverterAttribute(GetType(ValidatedControlConverterPlus))> _
    Public Property ControlToValidate4() As String
        Get
            Return _ControlToValidate(4)
        End Get
        Set(ByVal value As String)
            _ControlToValidate(4) = value
            BuildControlsToValidateString()
        End Set
    End Property

    ''' <summary>
    ''' Sixth control to validate.
    ''' </summary>
    <Browsable(True)> _
    <TypeConverterAttribute(GetType(ValidatedControlConverterPlus))> _
    Public Property ControlToValidate5() As String
        Get
            Return _ControlToValidate(5)
        End Get
        Set(ByVal value As String)
            _ControlToValidate(5) = value
            BuildControlsToValidateString()
        End Set
    End Property

    ''' <summary>
    ''' Seventh control to validate.
    ''' </summary>
    <Browsable(True)> _
    <TypeConverterAttribute(GetType(ValidatedControlConverterPlus))> _
    Public Property ControlToValidate6() As String
        Get
            Return _ControlToValidate(6)
        End Get
        Set(ByVal value As String)
            _ControlToValidate(6) = value
            BuildControlsToValidateString()
        End Set
    End Property

    ''' <summary>
    ''' Eigth control to validate.
    ''' </summary>
    <Browsable(True)> _
    <TypeConverterAttribute(GetType(ValidatedControlConverterPlus))> _
    Public Property ControlToValidate7() As String
        Get
            Return _ControlToValidate(7)
        End Get
        Set(ByVal value As String)
            _ControlToValidate(7) = value
            BuildControlsToValidateString()
        End Set
    End Property

    ''' <summary>
    ''' Ninth control to validate.
    ''' </summary>
    <Browsable(True)> _
    <TypeConverterAttribute(GetType(ValidatedControlConverterPlus))> _
    Public Property ControlToValidate8() As String
        Get
            Return _ControlToValidate(8)
        End Get
        Set(ByVal value As String)
            _ControlToValidate(8) = value
            BuildControlsToValidateString()
        End Set
    End Property

    ''' <summary>
    ''' Tenth control to validate.
    ''' </summary>
    <Browsable(True)> _
    <TypeConverterAttribute(GetType(ValidatedControlConverterPlus))> _
    Public Property ControlToValidate9() As String
        Get
            Return _ControlToValidate(9)
        End Get
        Set(ByVal value As String)
            _ControlToValidate(9) = value
            BuildControlsToValidateString()
        End Set
    End Property

    Private Function BuildControlsToValidateString()
        _ControlsToValidate = ""
        For Each control As String In _ControlToValidate
            If Not String.IsNullOrEmpty(control) Then
                _ControlsToValidate += control + "," 'don't worry about trailing comma
            End If
        Next control
        SetBaseControlToValidate()
        Return _ControlsToValidate
    End Function

    Private Sub SetBaseControlToValidate()
        Dim idx As Integer = _ControlsToValidate.IndexOf(",")
        If idx > 1 Then
            MyBase.ControlToValidate = _ControlsToValidate.Substring(0, idx)
        End If
    End Sub
#End Region

    Protected Overrides Sub OnPreRender(ByVal e As System.EventArgs)
        MyBase.OnPreRender(e)
        If RenderUplevel AndAlso Page IsNot Nothing AndAlso Not Page.ClientScript.IsClientScriptIncludeRegistered(Me.GetType(), "MissingValidator") Then
            Page.ClientScript.RegisterClientScriptInclude(Me.GetType(), "MissingValidator", Page.ClientScript.GetWebResourceUrl(Me.GetType(), "MissingValidator.AtLeastOneOfValidator.js"))
        End If
    End Sub

    Protected Overrides Sub AddAttributesToRender(ByVal writer As System.Web.UI.HtmlTextWriter)
        MyBase.AddAttributesToRender(writer)

        If EnableClientScript AndAlso RenderUplevel Then 'add client-side code if needed
            If EnableLegacyRendering() Then
                writer.AddAttribute("evaluationfunction", "ValidateAtLeastOneOf", False)
                writer.AddAttribute("controlstovalidate", ControlsToValidate, False)
            Else
                Page.ClientScript.RegisterExpandoAttribute(ClientID, "evaluationfunction", "ValidateAtLeastOneOf", False)
                Page.ClientScript.RegisterExpandoAttribute(ClientID, "controlstovalidate", ControlsToValidate, False)
            End If
        End If
    End Sub

    ''' <summary>
    ''' Check if we should use .AddAttribute() or an Expando attribute
    ''' </summary>
    ''' <returns>True if .AddAttribute(), false if Expando attribute</returns>
    ''' <remarks>For now there is only one control in the library.  If we add more, move this to a separate module that can be shared among them.</remarks>
    Private Shared Function EnableLegacyRendering() As Boolean
        Try
            Dim webConfigFile As String = IO.Path.Combine(HttpContext.Current.Request.PhysicalApplicationPath, "web.config")
            Dim webConfigReader As New XmlTextReader(New IO.StreamReader(webConfigFile))

            If webConfigReader.ReadToFollowing("xhtmlConformance") AndAlso webConfigReader.GetAttribute("mode") = "Legacy" Then
                EnableLegacyRendering = True
            Else
                EnableLegacyRendering = False
            End If
            webConfigReader.Close()
        Catch
            Return False
        End Try
    End Function
End Class